
/* 
 * JOGO DA VELHA na TV  -  Eletrogate
 * 
 * Criado por Etienne Leite Gomide
 * 02/08/2023
 * 
 */


#include <TVout.h>
#include <fontALL.h>
#include "TecladoMatricial.h";

TVout tv;
int marcaAtual;  //              0 = 'O', 1 = 'X'
int marcas[9];   // -1 = 'nulo', 0 = 'O', 1 = 'X'
bool fimDeJogo;

void setup()
{
  tv.begin(NTSC, 128,90);
  iniciarTecladoMatricial(12,11,10,  4,3,2);
  reiniciarJogo();
}

// Reseta a matriz de marcações 
void reiniciarJogo()
{
  // define a variável como FALSE para indicar que o jogo atual não está finalizado
  fimDeJogo = false;
  
  // define o tipo de marcaAtual como 'X'
  marcaAtual = 1;
  
  // coloca todos os índices da matriz como '-1'
  for (int i = 0; i < 9; i++) {
    marcas[i] = -1;
  }
}



int last_pos, pos;

void loop()
{
  last_pos = pos;
  pos = lerTecladoMatricial() - 1;

  if ((marcas[pos] == -1) && (pos >= 0) && (pos != last_pos))
  {
    // Coloca a marca atual na posição indicada pelo teclado Matricial.
    marcas[pos] = marcaAtual;

    // Inverte o tipo de marca a ser plotado no momento
    marcaAtual = 1 - marcaAtual;
    
    delay(50);
  }

  desenharTabuleiro();
  conferirVitoriaOuEmpate();

  // Se ainda não ocorreu o fim do jogo, então escreve o texto
  // que indica de quem é a vez de jogar.  
  if (fimDeJogo == false) {
    // muda a fonte e escreve o texto "Jogador".
    tv.select_font(font6x8);
    tv.print(86,5, "Jogador");

    // muda a fonte e escreve "O" ou "X" dependendo do tipo da marcaAtual.
    tv.select_font(font8x8ext);
    if(marcaAtual == 0) {
      tv.print(102,18, "O");
    } else
    if(marcaAtual == 1) {
      tv.print(102,18, "X");
    }
  }
  
  tv.delay_frame(1);
}


/* Desenha todos os elementos do tabuleiro:
 * linhas verticais, horizontais e marcações de "X" e "O".
 */
void desenharTabuleiro()
{
  int x, y;
  
  // Limpa a tela
  tv.clear_screen();

  // Desenha as linhas verticais e horizontais do tabuleiro
  tv.draw_rect(28,5, 1, 79, 1);
  tv.draw_rect(55,5, 1, 79, 1);
  tv.draw_rect(3,30, 79, 1, 1);
  tv.draw_rect(3,57, 79, 1, 1);

  // Desenha as marcações de 'X' e 'O'
  for (int i = 0; i < 9; i++)
  {
    // Seta a posição X,Y da marca a ser desenhada
    // posições da linha 1
    x = 7 + 27*i;
    y = 9;

    // posições da linha 2
    if(i > 2) {
      x = 7 + 27*i - 27*3;
      y = 9 + 27*1;
    }

    // posições da linha 3
    if(i > 5) {
      x = 7 + 27*i - 27*6;
      y = 9 + 27*2;
    }

    // Desenha a marca na posição X,Y determinada
    if(marcas[i] == 0) {
      // desenha a marca 'O'
      tv.draw_circle(x+8, y+8, 8, 1);
    } else
    if(marcas[i] == 1) {
      // desenha a marca 'X'
      tv.draw_line(x+1, y+1,  x+15,y+15, 1);
      tv.draw_line(x+1, y+15, x+15,y+1,  1);
    }
  }
}


/* Verifica se houve algum vencedor ou se
 * ocorreu um empate entre os jogadores.
 */
void conferirVitoriaOuEmpate()
{
  int vencedor = -1;  // 0 = Empate,  1 = houve vencedor
  int tempoPausa = 800;
  
  // verificar COLUNAS
  if ((marcas[0] >= 0) && (marcas[0] == marcas[3]) && (marcas[0] == marcas[6])) {
    tv.delay(tempoPausa);
    int x = 15 + 27*0;
    tv.draw_rect(x-1,6, 2, 79, 0,1);
    vencedor = 1;
  } else
  if ((marcas[1] >= 0) &&(marcas[1] == marcas[4]) && (marcas[1] == marcas[7])) {
    tv.delay(tempoPausa);
    int x = 15 + 27*1;
    tv.draw_rect(x-1,6, 2, 79, 0,1);
    vencedor = 1;
  } else
  if ((marcas[2] >= 0) &&(marcas[2] == marcas[5]) && (marcas[2] == marcas[8])) {
    tv.delay(tempoPausa);
    int x = 15 + 27*2;
    tv.draw_rect(x-1,6, 2, 79, 0,1);
    vencedor = 1;
  }

  // verificar LINHAS
  if ((marcas[0] >= 0) && (marcas[0] == marcas[1]) && (marcas[0] == marcas[2])) {
    tv.delay(tempoPausa);
    int y = 17 + 27*0;
    tv.draw_rect(4,y-1, 79,2, 0,1);
    vencedor = 1;
  } else
  if ((marcas[3] >= 0) && (marcas[3] == marcas[4]) && (marcas[3] == marcas[5])) {
    tv.delay(tempoPausa);
    int y = 17 + 27*1;
    tv.draw_rect(4,y-1, 79,2, 0,1);
    vencedor = 1;
  } else
  if ((marcas[6] >= 0) && (marcas[6] == marcas[7]) && (marcas[6] == marcas[8])) {
    tv.delay(tempoPausa);
    int y = 17 + 27*2;
    tv.draw_rect(4,y-1, 79,2, 0,1);
    vencedor = 1;
  }

  // Verificar DIAGONAIS
  if ((marcas[0] >= 0) && (marcas[0] == marcas[4]) && (marcas[0] == marcas[8])) {
    tv.delay(tempoPausa);
    tv.draw_line(6,8-2, 78,80-2, 0);
    tv.draw_line(6,8, 78,80, 1);
    tv.draw_line(6,8+2, 78,80+2, 0);
    vencedor = 1;
  } else
  if ((marcas[2] >= 0) && (marcas[2] == marcas[4]) && (marcas[2] == marcas[6])) {
    tv.delay(tempoPausa);
    tv.draw_line(6,80-2, 78,8-2, 0);
    tv.draw_line(6,80, 78,8, 1);
    tv.draw_line(6,80+2, 78,8+2, 0);
    vencedor = 1;
  }

  // Houve EMPATE ???
  // - - - - - - - - - - 
  // Verifica se toda a matriz já foi preenchida, para determinar se houve empate ou não.
  bool matrizPreenchida = true;
  for (int i = 0; i < 9; i++) {
    if(marcas[i] == -1) {
      matrizPreenchida = false;
      break;
    }
  }

  // Se a matriz estiver preenchida e não houve vencedor, então é declarado o EMPATE!
  if(matrizPreenchida && (vencedor == -1)) {
    tv.delay(tempoPausa);
    vencedor = 0;
    tv.select_font(font6x8);
    tv.print(90,40, "EMPATE");
    tv.delay(2000);
  }

  // Se houve um vencedor ou se ocorreu um empate, então é considerado Fim-De-Jogo.
  fimDeJogo = (vencedor >= 0);

  // Se houver uma vitória ou um empate, o jogo aguarda alguns segundos, limpa a tela e reinicia a partida.
  if(fimDeJogo) {
    tv.delay(8000);
    tv.clear_screen();
    tv.delay(tempoPausa);
    reiniciarJogo();
  }  
}





  
